@@ -78,15 +78,23 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <linux/types.h>
 
 #include "nls.h"
+#include "blkdev.h"
 #include "xstrncpy.h"
 #include "common.h"
 #include "gpt.h"
 
+#ifdef __GNU__
+#define DEFAULT_DEVICE "/dev/hd0"
+#define ALTERNATE_DEVICE "/dev/sd0"
+#elif defined(__FreeBSD__)
+#define DEFAULT_DEVICE "/dev/ad0"
+#define ALTERNATE_DEVICE "/dev/da0"
+#else
 #define DEFAULT_DEVICE "/dev/hda"
 #define ALTERNATE_DEVICE "/dev/sda"
+#endif
 
 /* With K=1024 we have `binary' megabytes, gigabytes, etc.
    Some misguided hackers like that.
@@ -180,8 +188,6 @@ int pt_heads = 0, pt_sectors = 0;
 
 static void
 set_hsc0(unsigned char *h, unsigned char *s, int *c, long long sector) {
-	if (sector >= 1024*cylinder_size)
-		sector = 1024*cylinder_size - 1;
 	*s = sector % sectors + 1;
 	sector /= sectors;
 	*h = sector % heads;
@@ -194,6 +200,8 @@ set_hsc(unsigned char *h, unsigned char *s, unsigned char *c,
 	long long sector) {
 	int cc;
 
+	if (sector >= 1024*cylinder_size)
+		sector = 1024*cylinder_size - 1;
 	set_hsc0(h, s, &cc, sector);
 	*c = cc & 0xFF;
 	*s |= (cc >> 2) & 0xC0;
@@ -391,7 +399,7 @@ partition_type_text(int i) {
 	 else
 	      return _("NTFS");
     } else
-	 return partition_type_name(p_info[i].id);
+	 return _(partition_type_name(p_info[i].id));
 }
 
 static void
@@ -1209,7 +1217,7 @@ menuSelect( int y, int x, struct MenuItem *menuItems, int itemLength,
     int i, ylast = y, key = 0, current = menuDefault;
 
     if( !( menuType & ( MENU_HORIZ | MENU_VERT ) ) ) {
-        print_warning(_("Menu without direction. Defaulting horizontal."));
+        print_warning(_("Menu without direction. Defaulting to horizontal."));
         menuType |= MENU_HORIZ;
     }
 
@@ -1687,7 +1695,7 @@ fill_p_info(void) {
     ioctl(fd, BLKFLSBUF);	/* ignore errors */
 				/* e.g. Permission Denied */
 
-    if (disksize(fd, &llsectors))
+    if (blkdev_get_sectors(fd, &llsectors) == -1)
 	    fatal(_("Cannot get disk size"), 3);
     actual_size = llsectors;
 
@@ -2067,7 +2075,7 @@ print_p_info_entry(FILE *fp, partition_info *p) {
     else if (p->id == FREE_SPACE)
 	sprintf(part_str, "%.15s", _("Free Space"));
     else if (partition_type_name(p->id))
-	sprintf(part_str, "%.15s (%02X)", partition_type_name(p->id), p->id);
+	sprintf(part_str, "%.15s (%02X)", _(partition_type_name(p->id)), p->id);
     else
 	sprintf(part_str, "%.15s (%02X)", _("Unknown"), p->id);
     fp_printf(fp, "%-20.20s", part_str);
@@ -2163,7 +2171,7 @@ print_part_entry(FILE *fp, int num, partition_info *pi) {
 	set_hsc0(&eh, &es, &ec, end);
     }
 
-    fp_printf(fp, "%2d  0x%02X %4d %4d %4d 0x%02X %4d %4d %4d %11lld %11lld\n",
+    fp_printf(fp, "%2d  0x%02X %4d %4d %5d 0x%02X %4d %4d %5d %11lld %11lld\n",
 	      num+1, flags, sh, ss, sc, id, eh, es, ec, first, size);
 }
 
@@ -2201,9 +2209,9 @@ print_part_table(void) {
     fp_printf(fp, _("Partition Table for %s\n"), disk_device);
     fp_printf(fp, "\n");
     /* Three-line heading. Read "Start Sector" etc vertically. */
-    fp_printf(fp, _("         ---Starting---      ----Ending----    Start     Number of\n"));
-    fp_printf(fp, _(" # Flags Head Sect Cyl   ID  Head Sect Cyl     Sector    Sectors\n"));
-    fp_printf(fp, _("-- ----- ---- ---- ---- ---- ---- ---- ---- ----------- -----------\n"));
+    fp_printf(fp, _("         ---Starting----      ----Ending-----    Start     Number of\n"));
+    fp_printf(fp, _(" # Flags Head Sect  Cyl   ID  Head Sect  Cyl     Sector    Sectors\n"));
+    fp_printf(fp, _("-- ----- ---- ---- ----- ---- ---- ---- ----- ----------- -----------\n"));
 
     for (i = 0; i < 4; i++) {
 	for (j = 0;
@@ -2483,7 +2491,7 @@ change_id(int i) {
 		move(row, ((j-1)/num_down)*COL_ID_WIDTH + 1);
 		printw("%02X %-20.20s",
 		       i386_sys_types[j].type,
-		       i386_sys_types[j].name);
+		       _(i386_sys_types[j].name));
 	    }
 	    if (row > row_max)
 		needmore = 1;
@@ -2766,15 +2774,15 @@ do_curses_fdisk(void) {
 
 	if (p_info[cur_part].id == FREE_SPACE) {
 	    s = ((opentype == O_RDWR) ? "hnpquW" : "hnpqu");
-	    command = menuSelect(COMMAND_LINE_Y, COMMAND_LINE_X, menuMain, 8,
+	    command = menuSelect(COMMAND_LINE_Y, COMMAND_LINE_X, menuMain, 10,
 	        s, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, 0);
 	} else if (p_info[cur_part].id > 0) {
 	    s = ((opentype == O_RDWR) ? "bdhmpqtuW" : "bdhmpqtu");
-	    command = menuSelect(COMMAND_LINE_Y, COMMAND_LINE_X, menuMain, 8,
+	    command = menuSelect(COMMAND_LINE_Y, COMMAND_LINE_X, menuMain, 10,
 	        s, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, 0);
 	} else {
 	    s = ((opentype == O_RDWR) ? "hpquW" : "hpqu");
-	    command = menuSelect(COMMAND_LINE_Y, COMMAND_LINE_X, menuMain, 8,
+	    command = menuSelect(COMMAND_LINE_Y, COMMAND_LINE_X, menuMain, 10,
 	        s, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, 0);
 	}
 	switch ( command ) {
